//+------------------------------------------------------------------+
//|                                                     mql4pack.mq4 |
//|                                    Copyright (c) 2020, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//|                               https://www.mql5.com/en/code/27955 |
//+------------------------------------------------------------------+
#property copyright "Copyright (c) 2020, Marketeer"
#property link "https://www.mql5.com/en/users/marketeer"
#property version "1.0"
#property description "This is MQL4 project packer: assemble all source and resource files from dependencies into a single ZIP."

#property script_show_inputs


// NB: first level folder name is assumed to be created virtual root (real-MQL5 container)
input string VirtualRootFolder = "MQL4";
// by default this script will pack itself with all dependencies
input string MainSourceFile = "Scripts/mql4pack.mq4";
// by default (if PackedFile is empty) zip-filename will be source-file-name.mq*.zip
input string PackedFile = "";
// optionally specify a text file with elements to exclude from process (1 element per line)
input string ExceptionListFilename = "MQLPack/mql4pack.txt";

string IncludesFolder; // must include trailing slash
const bool LoadIncludes = true;

// Basic support for icons
// Please load this file from the page https://www.mql5.com/en/code/27955
// Strangely, the codebase does not support ico/png files uploading
#property icon "\\Images\\mqlpack.png"

// Basic support for Images and Sounds (uncomment for demonstration purpose only)
// #resource "\\Images\\euro.bmp";
// #resource "\\Images\\dollar.bmp";

// this is shell command to create symlink to terminal's data folder with
// all source and resource folders inside MQL5/Files/[pseudo-MQL5-link]
// rename this txt-file to makelink.bat (bat-files can not be added as resources!)
//#resource "\\Files\\makelink.bat" as string __cmd1Folder

// example with exception list (standard included files)
//#resource "\\Files\\mql4pack.txt" as string __exceptionListExample


// #define UNICODE_WARNING
#include <Codebase/FileReader.mqh>
#include <Zip/Zip.mqh>


CZip Zip;      // Create empty zip archive.
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnStart()
 {
  Print("\n");

  string exceptions[];
  int ex = 0;

  if(StringLen(ExceptionListFilename) > 0)
   {
    int h = FileOpen(ExceptionListFilename, FILE_READ | FILE_TXT | FILE_ANSI | FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_COMMON, 0, CP_UTF8);
    if(h == INVALID_HANDLE)
     {
      Print("Can't open ", ExceptionListFilename, " ", GetLastError());
     }
    else
     {
      while(!FileIsEnding(h))
       {
        string line = FileReadString(h);
        if(StringLen(line) > 0)
         {
          StringToLower(line);
          ArrayResize(exceptions, ex + 1);
          exceptions[ex++] = line;
         }
       }
      FileClose(h);
     }
   }

  string root = VirtualRootFolder;
  StringToLower(root);

  IncludesFolder = VirtualRootFolder + "/Include/";

  const string SourceFile = VirtualRootFolder + "/" + MainSourceFile;
  Preprocessor loader(SourceFile, IncludesFolder, LoadIncludes);

  Print("Scanning...");

  if(!loader.run())
   {
    Print("Loader failed");
    return;
   }

  Print("Files processed: ", loader.scannedFilesCount());
  Print("Source length: ", loader.totalCharactersCount());

  Print("Packing...");
  int added = 0, j;
  const uint n = loader.scannedFilesCount();
  string listing = "";

  for(uint i = 0; i < n; i++)
   {
    string name = loader.scannedFilename(i);
    for(j = 0; j < ex; j++)
     {
      string _name = name;
      StringToLower(_name);
      if(StringFind(_name, root + "/" + exceptions[j]) == 0)
       {
        Print(" - ", name);
        listing += " - " + name + "\n";
        break;
       }
     }

    if(j < ex) continue;

    Print(" + ", name);
    listing += " + " + name + "\n";
    CZipFile *file = new CZipFile();
    if(!file.AddFile(name,0)
        || !Zip.AddFile(file))
     {
      delete file;
     }
    else
     {
      added++;
     }
   }

  if(added > 0)
   {
    string dirfile = SourceFile + ".log";
    StringReplace(dirfile, "/", "_");
    uchar array[];
    StringToCharArray(listing, array, 0, WHOLE_ARRAY, CP_UTF8);
    CZipFile *dir = new CZipFile(dirfile, array);
    Zip.AddFile(dir);

    const string archive = (PackedFile != "" ? PackedFile : SourceFile) + ".zip";
    if(Zip.SaveZipToFile(archive,0))
     {
      Print("Packed file saved: ", archive);
     }
   }
  else
   {
    Print("No files added.");
    Print("Make sure to create pseudo-MQL4-link in the real MQL4/Files folder by running makelink.bat:");
    //Print(__cmd1Folder);
   }
 }

//+------------------------------------------------------------------+
